/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <ctype.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "kernelconf.h"
#include "expressions.h"
#include "utils.h"
#include "tree.h"

extern IdPtr id_tree_root;

/*
 * Evaluates a list of expressions pointed by EXPRESSION and retuns zero if any
 * of them evaluates to n, y otherwise.  Empty expression lists are accepted.
 */
int evaluate_all_expressions(ExpressionPtr expression)
{
    TokenPtr token;
    ExpressionPtr e;
    char c;

    for (e = expression; e; e = e->next) {
	token = evaluate_expression(e);
	c = get_boolean_value(token);
	free_token(token);
	if ((c == 0) || (c == 'n'))
	    return 0;
    }
    return 1;
}

void set_derived_value(IdPtr id)
{
    TokenPtr token;
    ExpressionPtr e;

    if (id->symbol == NULL)
	return;

    for (e = id->symbol->derived; e; e = e->next) {
	token = evaluate_expression(e);
	if (token->type != TYPE_NULL) {
	    token->type = get_generic_type(token->type);
	    switch (id->symbol->type) {
		case TYPE_BOOL:
		case TYPE_TRI:
		    id->symbol->value.bool_val = get_boolean_value(token);
		    break;
		case TYPE_INT:
		    if (token->type == TYPE_INT)
			id->symbol->value.int_val = token->value.int_val;
		    else if (token->type == TYPE_VALUE)
			id->symbol->value.int_val = atoi(token->value.str_val);
		    break;
		default:
		    break;
	    }
	    free_token(token);
	    return;
	}
	free_token(token);
    }
}

static IdListPtr add_id_to_list(IdPtr id, IdListPtr list)
{
    IdListPtr i;

    i = list;
    if (i) {
	do {
	    if (i->id == id)
		return list;
	    if (i->next == NULL)
		break;
	    i = i->next;
	} while (1);
	i->next = (IdListPtr)malloc(sizeof(IdList));
	i = i->next;
	i->id = id;
	i->next = NULL;
    }
    else {
	list = (IdListPtr)malloc(sizeof(IdList));
	list->id = id;
	list->next = NULL;
    }
    return list;
}

static IdListPtr get_id_list(ExpressionPtr expression, IdListPtr list)
{
    ExpressionPtr e;
    TokenPtr t;

    for (e = expression; e; e = e->next)
	for (t = e->first_token; t; t = t->next)
	    if (t->type == TYPE_SYMBOL)
		list = add_id_to_list(t->value.id_val, list);
    return list;
}

/*
 * Adds ID to affected lists of ids in depends/menu-depends/derived fields
 */ 
int set_affected_ids(IdPtr id)
{
    IdListPtr list, i;

    list = NULL;
    if (id->symbol) {
	list = get_id_list(id->symbol->depends, list);
	list = get_id_list(id->symbol->derived, list);
    }
    if (id->menu)
	list = get_id_list(id->menu->depends, list);
    for (i = list; i; i = i->next)
	i->id->symbol->affects = add_id_to_list(id, i->id->symbol->affects);
    return 0;
}

