/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#ifndef _KCONF_H
#define _KCONF_H

/* data types */
typedef enum {
    TYPE_NULL,
    TYPE_BOOL,
    TYPE_TRI,
    TYPE_INT,
    TYPE_HEX,
    TYPE_CHAR,
    TYPE_STR,
    TYPE_OPTION,
    TYPE_UNKNOWN,
    TYPE_DEPENDS,
    TYPE_DERIVED,
    TYPE_OR,
    TYPE_AND,
    TYPE_EQUAL,
    TYPE_NOT_EQUAL,
    TYPE_LT,
    TYPE_GT,
    TYPE_LTE,
    TYPE_GTE,
    TYPE_NOT,
    TYPE_LEFT_PARA,
    TYPE_RIGHT_PARA,
    TYPE_SYMBOL,
    TYPE_VALUE
} Type;

/* Precedence is NULL for non-operators */
typedef enum {
    PRE_NULL,
    PRE_ASSIGN,
    PRE_OR,
    PRE_AND,
    PRE_EQUAL,
    PRE_REL,
    PRE_NOT
} Precedence;

/* tags in config files */
typedef enum {
    TAG_NULL,
    TAG_EOF,
    TAG_BLANK,
    TAG_INCLUDE,
    TAG_SYMBOL,
    TAG_TYPE,
    TAG_VALUE,
    TAG_DEFAULT,
    TAG_DESC,
    TAG_DEPENDS,
    TAG_DERIVED,
    TAG_DESC_LINE,
    TAG_MENU_ITEM,
    TAG_MENU_CAPTION,
    TAG_MENU_PARENT,
    TAG_MENU_DEPENDS
} Tag;

typedef struct _Id* IdPtr;
typedef struct _IdList* IdListPtr;
typedef struct _Symbol* SymbolPtr;
typedef struct _Menu* MenuPtr;

typedef union _Value {
    char char_val;
    char bool_val;
    int int_val;
    char *str_val;
    void *ptr_val;
    IdPtr id_val;
} Value;

typedef struct _Token* TokenPtr;

typedef struct _Token {
    Type type;
    Precedence precedence;
    Value value;
    TokenPtr next;
} Token;

typedef struct _Expression* ExpressionPtr;

typedef struct _Expression {
    TokenPtr first_token;
    char *string;
    ExpressionPtr next;
} Expression;

/* 
 * An identifier contains a symbol and/or a menu.
 */

typedef struct _Id {
    char *name;
    IdPtr left, right;
    SymbolPtr symbol;
    MenuPtr menu;
    char *config_file;
    long desc_start, desc_length;
    unsigned int flags;
} Id;

typedef struct _IdList{
    IdPtr id;
    IdListPtr next;
} IdList;

typedef struct _Symbol {
    Type type;
    Value value;
    ExpressionPtr depends, derived;
    IdListPtr affects;
    IdPtr next;
    unsigned int flags;
} Symbol;

typedef struct _Menu {
    IdPtr parent, first_child, last_child, next;
    char *caption;
    ExpressionPtr depends;
    unsigned int flags;
} Menu;

/* identifier flags */
/* two bits for balance factor */
#define ID_FLAG_BF    3
#define ID_FLAG_MARK  4

/* search types */
typedef enum {
    ID_SEARCH,
    ID_INSERT,
    ID_MARK_INSERT
} Search;

/* menu flags */
#define MENU_FLAG_OK           0x01   /* is set if menu is fully initialized */
#define MENU_FLAG_VISIBLE      0x02
#define MENU_FLAG_EXPANDED     0x04
#define MENU_FLAG_VALID        0x08
#define MENU_FLAG_DEPENDS_OK   0x10
#define MENU_FLAG_SEEN         0x20
#define MENU_FLAG_INTERNAL     0x40
#define MENU_FLAG_SUBS_ON      0x80   /* is set if any of the sub menu
					 items are on */

/* symbol flags */
#define SYMBOL_FLAG_OK         0x01   /* is set if symbol is fully initialized */
#define SYMBOL_FLAG_FROZEN     0x02
#define SYMBOL_FLAG_DEPENDS_OK 0x04
#define SYMBOL_FLAG_NOT_SET    0x08   /* is set if symbol is altered */
#define SYMBOL_FLAG_CHANGED    0x10   /* is set if symbol is altered */
#define SYMBOL_FLAG_INTERNAL   0x20   /* is set if symbol is altered */

#endif /* _KCONF_H */

