/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <malloc.h>

#include "tree.h"
#include "utils.h"

/*
 * Create a new identifier and initialize its fields
 */
static IdPtr new_node(char *key)
{
    IdPtr tmp;

    tmp = (IdPtr)malloc(sizeof(Id));
    tmp->name = string_duplicate(key);
    tmp->left = NULL;
    tmp->right = NULL;
    tmp->symbol = NULL;
    tmp->menu = NULL;
    tmp->config_file = NULL;
    tmp->desc_start = -1;
    tmp->desc_length = 0;
    tmp->flags = 0;

    return (tmp);
}

/*
 * looks for the string KEY in the tree pointed by ROOT and if it is not found,
 * inserts to the tree if INSERT is set to ID_INSERT or ID_MARK_INSERT, or
 * returns NULL if INSERT is set to ID_SEARCH.  In ID_MARK_INSERT case, the
 * ID_FLAG_MARK is set to indicate that the node is not initialized.  This flag
 * is cleared in a later ID_INSERT request for the same key.
 *
 */
IdPtr search_tree(IdPtr *root, char *key, Search search)
{
    IdPtr ptr, balance, parent, new;
    int cmp;

    if ((*root) == NULL) {
	if (search == ID_SEARCH)
	    return NULL;
	new = new_node(key);
	if (search == ID_MARK_INSERT)
	    new->flags = ID_FLAG_MARK;
	*root = new;
	return new;
    }

    ptr = *root;
    balance = *root;
    parent = *root;

    while (1) {
	cmp = strcmp(key, ptr->name);

	if (cmp < 0) {
	    if (ptr->left) {
		if ((ptr->flags & ID_FLAG_BF) - 1) {
		    parent = ptr;
		    balance = ptr->left;
		}
		ptr = ptr->left;
	    }
	    else {
		if (search == ID_SEARCH)
		    return NULL;
		new = new_node(key);
		ptr->left = new;
		break;
	    }
	}
	else if (cmp > 0) {
	    if (ptr->right) {
		if ((ptr->flags & ID_FLAG_BF) - 1) {
		    parent = ptr;
		    balance = ptr->right;
		}
		ptr = ptr->right;
	    }
	    else {
		if (search == ID_SEARCH)
		    return NULL;
		new = new_node(key);
		ptr->right = new;
		break;
	    }
	}
	else {
	    /* found */
	    if ((search == ID_INSERT) && (ptr->flags & ID_FLAG_MARK))
		ptr->flags -= ID_FLAG_MARK;
	    return ptr;
	}
    }

    /* we have just inserted a new node at NEW */
    if (search == ID_MARK_INSERT)
	new->flags = ID_FLAG_MARK;

    return new;

    /* FIXME: balance the tree */

    return NULL;
}

