/*
 * KernelConf - A Linux Kernel Configuration System
 *
 * Distributed under GNU General Public Lisence
 *
 * Copyright (C) 2002 by Anuradha Ratnaweera
 */

#include <ctype.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>

#include "tree.h"
#include "symbols.h"
#include "writeconfig.h"

extern MenuPtr root_menu_item;

static int read_char_input()
{
    char c;

    do {
	if ((c = getchar()) == '\n')
	    return c;
    } while (isspace(c));

    while (getchar() != '\n')
	;

    return c;
}

static int read_int_input()
{
    int i;

    scanf("%d", &i);

    /* while (getchar() != '\n')
	; */

    return i;
}

static int show_help(IdPtr id)
{
    char *buffer;
    long bufsize;

    if ((bufsize = read_help(id, &buffer)) < 0)
	return 1;

    fwrite(buffer, sizeof(char), bufsize, stdout);
    free(buffer);

    return 0;
}

void do_menu(IdPtr parent)
{
    int c, c_in;
    char ok, selected = 0;
    MenuPtr menu_item, option;
    int i, i_in;

    menu_item = parent->child;

    do {
	ok = 0;

	if (menu_item->child && (parent == root_menu_item)) {
	    printf("*\n");
	    printf("* %s\n", menu_item->caption);
	    printf("*\n");
	}

	if (menu_item->symbol) {
	    switch (menu_item->symbol->type) {
		case TYPE_BOOL:
		    c = menu_item->symbol->value.charval;
		    printf("%s (CONFIG_%s) [%c/%c/?] ", menu_item->caption,
			    ((NodePtr)menu_item->symbol)->name,
			    (c == 'n') || (c == 0) ? 'N' : 'n',
			    c == 'y' ? 'Y' : 'y');
		    c_in = tolower(read_char_input());
		    if ((c_in == 'y') || ((c == 'y') && (c_in = '\n')))
			selected = 1;
		    if ((c_in == 'y') || (c_in == 'n')) {
			menu_item->symbol->value.charval = tolower(c_in);
			ok = 1;
		    }
		    else if (c_in == '\n')
			ok = 1;
		    else
			show_help(menu_item);
		    break;
		case TYPE_TRI:
		    c = menu_item->symbol->value.charval;
		    printf("%s (CONFIG_%s) [%c/%c/%c/?] ", menu_item->caption,
			    ((NodePtr)menu_item->symbol)->name,
			    (c == 'n') || (c == 0) ? 'N' : 'n',
			    c == 'm' ? 'M' : 'm',
			    c == 'y' ? 'Y' : 'y');
		    c_in = tolower(read_char_input());
		    if ((c_in == 'y') || (c_in == 'm') ||
			    (((c == 'y') || (c == 'm')) && (c_in = '\n')))
			selected = 1;
		    if ((c_in == 'y') || (c_in == 'm') || (c_in == 'n')) {
			menu_item->symbol->value.charval = tolower(c_in);
			ok = 1;
		    }
		    else if (c_in == '\n')
			ok = 1;
		    else
			show_help(menu_item);
		    break;
		case TYPE_INT:
		case TYPE_HEX:
		    break;
		case TYPE_CHAR:
		    break;
		case TYPE_STR:
		    break;
		case TYPE_OPTION:
		    printf("%s\n", menu_item->parent->caption);
		    for (i = 1, option = menu_item; option; i++,
			    option = (MenuPtr)(((NodePtr)option)->next)) {
			printf("(%d) %s", i, option->caption);
			if (option->symbol->value.charval == 'y') {
			    printf(" (*)");
			    i_in = i;
			}
			printf("\n");
		    }
		    printf("Select (1 - %d) [%d] ", i - 1, i_in);
		    i_in = read_int_input();
		    if ((i_in > 0) && (i_in < i)) {
			for (i = 1, option = menu_item; option; i++,
				option = (MenuPtr)(((NodePtr)option)->next)) {
			    if (i == i_in) {
				option->symbol->value.charval = 'y';
			    }
			    else {
				option->symbol->value.charval = 'n';
			    }
			}
		        ok = 1;
			menu_item = NULL;
		    }
		    else {
			/* isn't this odd ? ;) */
			show_help (menu_item);
		    }
		    break;
		default:
		    ;
	    }
	}
	else {
	    ok = 1;
	    selected = 1;
	}

	if (ok && menu_item) {
	    if (menu_item->child && selected)
		do_menu(menu_item);
	    menu_item = (MenuPtr)((NodePtr)menu_item)->next;
	    selected = 0;
	}
    } while (menu_item);
}

int do_ttyconfig()
{
    int c;

    if ((root_menu_item == NULL) || (root_menu_item->child == NULL))
	return 1;

    do_menu(root_menu_item);

    do {
	printf("Do you wish to save your new kernel configuration [Y/n] ");
	c = tolower(read_char_input());
	if ((c == 'y') || (c == '\n')) {
	    printf("\n");
	    printf("Saving your kernel configuration...");
	    write_config(".config", "include/linux/autoconf.h");
	    printf("done.\n");
	    printf("\n");
	    printf("*** End of Linux kernel configuration.\n");
	    printf("*** Check the top-level Makefile for additional configuration.\n");
	    printf("*** Next, you must run 'make dep'.\n");
	    printf("\n");
	}
	else if (c == 'n') {
	    printf("\n");
	    printf("Your kernel configuration changes were NOT saved.\n");
	    printf("\n");
	}
	else
	    c = 0;
    } while (c == 0);

    return 0;
}

